@@ -1,4 +1,4 @@
-// Copyright 2009-2020 Intel Corporation
+// Copyright 2009-2021 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #include "scene_user_geometry.h"
@@ -34,7 +34,16 @@ namespace embree
   void UserGeometry::setOccludedFunctionN (RTCOccludedFunctionN occluded) {
     intersectorN.occluded = occluded;
   }
-  
+
+  size_t UserGeometry::getGeometryDataDeviceByteSize() const {
+    return 16 * ((sizeof(UserGeometry) + 15) / 16);
+  }
+
+  void UserGeometry::convertToDeviceRepresentation(size_t offset, char* data_host, char* data_device) const {
+    std::memcpy(data_host + offset, (void*)this, sizeof(UserGeometry));
+    offset += sizeof(Instance);
+  }
+
 #endif
 
   namespace isa
