@@ -9,7 +9,7 @@ namespace embree
 {
   namespace SceneGraph
   {
-    void storeXML(Ref<SceneGraph::Node> root, const FileName& fileName, bool embedTextures, bool referenceMaterials);
+    void storeXML(Ref<SceneGraph::Node> root, const FileName& fileName, bool embedTextures, bool referenceMaterials, bool binaryFormat);
   }
 }
 
