@@ -61,19 +61,20 @@ namespace embree
 #endif
   };
 
-  enum ISPCType { TRIANGLE_MESH, SUBDIV_MESH, CURVES, INSTANCE, GROUP, QUAD_MESH, GRID_MESH, POINTS };
+  enum ISPCType { TRIANGLE_MESH, SUBDIV_MESH, CURVES, INSTANCE, INSTANCE_ARRAY, GROUP, QUAD_MESH, GRID_MESH, POINTS };
 
   struct ISPCGeometry
   {
 #if !defined(ISPC)
-    ISPCGeometry (ISPCType type) : type(type), geometry(nullptr), scene(nullptr), geomID(-1), materialID(-1) {}
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCGeometry (ISPCType type) : type(type), geometry(nullptr), materialID(-1), visited(false) {}
     ~ISPCGeometry () { if (geometry) rtcReleaseGeometry(geometry); }
 #endif
     ISPCType type;
     RTCGeometry geometry;
-    RTCScene scene;
-    unsigned int geomID;
     unsigned int materialID;
+    bool visited;
   };
 
 #if !defined(ISPC)
@@ -87,9 +88,14 @@ namespace embree
   struct ISPCTriangleMesh
   {
 #if !defined(ISPC)
-    ISPCTriangleMesh (TutorialScene* scene_in, Ref<SceneGraph::TriangleMeshNode> in);
+    ALIGNED_STRUCT_USM_(16);
+
+    ISPCTriangleMesh (RTCDevice device, unsigned int numTriangles, unsigned int numPositions, bool hasNormals, bool hasTexcoords, unsigned int numTimeSteps = 1);
+    ISPCTriangleMesh (RTCDevice device, TutorialScene* scene_in, Ref<SceneGraph::TriangleMeshNode> in);
     ~ISPCTriangleMesh ();
 
+    void commit();
+
   private:
     ISPCTriangleMesh (const ISPCTriangleMesh& other) DELETED; // do not implement
     ISPCTriangleMesh& operator= (const ISPCTriangleMesh& other) DELETED; // do not implement
@@ -113,9 +119,13 @@ namespace embree
   struct ISPCQuadMesh
   {
 #if !defined(ISPC)
-    ISPCQuadMesh (TutorialScene* scene_in, Ref<SceneGraph::QuadMeshNode> in);
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCQuadMesh (RTCDevice device, TutorialScene* scene_in, Ref<SceneGraph::QuadMeshNode> in);
     ~ISPCQuadMesh ();
 
+    void commit();
+
   private:
     ISPCQuadMesh (const ISPCQuadMesh& other) DELETED; // do not implement
     ISPCQuadMesh& operator= (const ISPCQuadMesh& other) DELETED; // do not implement
@@ -139,8 +149,12 @@ namespace embree
   struct ISPCSubdivMesh
   {
 #if !defined(ISPC)
-    ISPCSubdivMesh (TutorialScene* scene_in, Ref<SceneGraph::SubdivMeshNode> in);
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCSubdivMesh (RTCDevice device, TutorialScene* scene_in, Ref<SceneGraph::SubdivMeshNode> in);
     ~ISPCSubdivMesh ();
+
+    void commit();
     
   private:
     ISPCSubdivMesh (const ISPCSubdivMesh& other) DELETED; // do not implement
@@ -184,9 +198,13 @@ namespace embree
   struct ISPCHairSet
   {
 #if !defined(ISPC)
-    ISPCHairSet (TutorialScene* scene_in, RTCGeometryType type, Ref<SceneGraph::HairSetNode> in);
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCHairSet (RTCDevice device, TutorialScene* scene_in, RTCGeometryType type, Ref<SceneGraph::HairSetNode> in);
     ~ISPCHairSet();
 
+    void commit();
+
   private:
     ISPCHairSet (const ISPCHairSet& other) DELETED; // do not implement
     ISPCHairSet& operator= (const ISPCHairSet& other) DELETED; // do not implement
@@ -218,9 +236,13 @@ namespace embree
   struct ISPCPointSet
   {
 #if !defined(ISPC)
-    ISPCPointSet (TutorialScene* scene_in, RTCGeometryType type, Ref<SceneGraph::PointSetNode> in);
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCPointSet (RTCDevice device, TutorialScene* scene_in, RTCGeometryType type, Ref<SceneGraph::PointSetNode> in);
     ~ISPCPointSet();
 
+    void commit();
+
   private:
     ISPCPointSet (const ISPCPointSet& other) DELETED; // do not implement
     ISPCPointSet& operator= (const ISPCPointSet& other) DELETED; // do not implement
@@ -242,9 +264,13 @@ namespace embree
   struct ISPCGridMesh
   {
 #if !defined(ISPC)
-    ISPCGridMesh (TutorialScene* scene_in, Ref<SceneGraph::GridMeshNode> in);
+    ALIGNED_STRUCT_USM_(16);
+    
+    ISPCGridMesh (RTCDevice device, TutorialScene* scene_in, Ref<SceneGraph::GridMeshNode> in);
     ~ISPCGridMesh ();
 
+    void commit();
+
   private:
     ISPCGridMesh (const ISPCGridMesh& other) DELETED; // do not implement
     ISPCGridMesh& operator= (const ISPCGridMesh& other) DELETED; // do not implement
@@ -267,9 +293,14 @@ namespace embree
   struct ISPCInstance
   {
 #if !defined(ISPC)
-    ISPCInstance (TutorialScene* scene, Ref<SceneGraph::TransformNode> in);
+    ALIGNED_STRUCT_USM_(16);
+
+    ISPCInstance (RTCDevice device, unsigned int numTimeSteps = 1);
+    ISPCInstance (RTCDevice device, TutorialScene* scene, Ref<SceneGraph::TransformNode> in);
     ~ISPCInstance();
 
+    void commit();
+
   private:
     ISPCInstance (const ISPCInstance& other) DELETED; // do not implement
     ISPCInstance& operator= (const ISPCInstance& other) DELETED; // do not implement
@@ -286,11 +317,44 @@ namespace embree
     AffineSpace3fa* spaces;
   };
 
+  struct ISPCInstanceArray
+  {
+#if !defined(ISPC)
+    ALIGNED_STRUCT_USM_(16);
+
+    ISPCInstanceArray(RTCDevice device, unsigned int numTimeSteps = 1);
+    ISPCInstanceArray(RTCDevice device, TutorialScene* scene, Ref<SceneGraph::MultiTransformNode> in);
+    ~ISPCInstanceArray();
+
+    void commit();
+
+  private:
+    ISPCInstanceArray(const ISPCInstanceArray& other) DELETED; // do not implement
+    ISPCInstanceArray& operator= (const ISPCInstanceArray& other) DELETED; // do not implement
+
+  public:
+#endif
+
+    ISPCGeometry geom;
+    ISPCGeometry* child;
+    float startTime;
+    float endTime;
+    unsigned int numTimeSteps;
+    unsigned int numInstances;
+    bool quaternion;
+    AffineSpace3fa** spaces_array;
+  };
+
   struct ISPCGroup
   {
 #if !defined(ISPC)
-    ISPCGroup (TutorialScene* scene, Ref<SceneGraph::GroupNode> in);
+    ALIGNED_STRUCT_USM_(16);
+
+    ISPCGroup (RTCDevice device,  unsigned int numGeometries);
+    ISPCGroup (RTCDevice device, TutorialScene* scene, Ref<SceneGraph::GroupNode> in);
     ~ISPCGroup();
+
+    void commit();
     
   private:
     ISPCGroup (const ISPCGroup& other) DELETED; // do not implement
@@ -299,18 +363,29 @@ namespace embree
   public:
 #endif
     ISPCGeometry geom;
+    RTCScene scene;
     ISPCGeometry** geometries;
     unsigned int numGeometries;
+    unsigned int requiredInstancingDepth; // instancing depth required for this group
   };
   
   struct ISPCScene
   {
 #if !defined(ISPC)
-    ISPCScene(TutorialScene* in);
+    ALIGNED_STRUCT_USM_(16);
+
+    ISPCScene(RTCDevice device, unsigned int numGeometries, unsigned int numMaterials, unsigned int numLights);
+    ISPCScene(RTCDevice device, TutorialScene* in);
     ~ISPCScene();
+
+    void commit();
     
-    static ISPCGeometry* convertGeometry (TutorialScene* scene, Ref<SceneGraph::Node> in);   
-    static Light* convertLight(Ref<SceneGraph::Light> in);
+    static ISPCGeometry* convertGeometry (RTCDevice device, TutorialScene* scene, Ref<SceneGraph::Node> in);   
+    static Light* convertLight(Ref<SceneGraph::LightNode> in);
+    static Light* createLight(Ref<SceneGraph::LightNode> in);
+
+    template<typename LightNode> static void updateLight(const LightNode& in, Light* out);
+    static void updateLight(const Ref<SceneGraph::LightNode>& in, Light* out);
     
   private:
     ISPCScene (const ISPCScene& other) DELETED; // do not implement
@@ -318,6 +393,7 @@ namespace embree
     
   public:
 #endif
+    RTCScene scene;
     ISPCGeometry** geometries;   //!< list of geometries
     ISPCMaterial** materials;     //!< material list
     unsigned int numGeometries;           //!< number of geometries
@@ -325,12 +401,23 @@ namespace embree
     
     Light** lights;              //!< list of lights
     unsigned int numLights;               //!< number of lights
+    void* tutorialScene;
   };
 
-#if !defined(ISPC)  
-  extern "C" RTCScene ConvertScene(RTCDevice g_device, ISPCScene* scene_in, RTCBuildQuality quality);
+#if !defined(ISPC)
+  typedef void (*AssignShaderTy)(ISPCGeometry* geometry);
+  extern "C" { extern AssignShaderTy assignShadersFunc; };
 #else
-  unmasked extern "C" RTCScene ConvertScene (RTCDevice g_device, ISPCScene* uniform scene_in, uniform RTCBuildQuality quality);
+  typedef unmasked void (*AssignShaderTy)(uniform ISPCGeometry* uniform geometry);
+  extern uniform AssignShaderTy assignShadersFunc;
+#endif
+  
+#if !defined(ISPC)
+  extern "C" void UpdateScene(ISPCScene* scene_in, float time);
+  extern "C" RTCScene ConvertScene(RTCDevice g_device, ISPCScene* scene_in, RTCBuildQuality quality, RTCSceneFlags flags = RTC_SCENE_FLAG_NONE, RTCFeatureFlags *used_features_out=nullptr);
+#else
+  unmasked extern "C" void UpdateScene(ISPCScene* uniform scene_in, uniform float time);
+  unmasked extern "C" RTCScene ConvertScene (RTCDevice g_device, ISPCScene* uniform scene_in, uniform RTCBuildQuality quality, uniform RTCSceneFlags flags = RTC_SCENE_FLAG_NONE, RTCFeatureFlags * uniform used_features_out=NULL);
 #endif
 
 #if !defined(ISPC)                    
