@@ -672,7 +672,7 @@ namespace embree
     this->height = height;
       
     if (pixels) alignedUSMFree(pixels);
-    pixels = (unsigned*) alignedUSMMalloc(width*height*sizeof(unsigned),64,EMBREE_USM_SHARED_DEVICE_READ_WRITE);
+    pixels = (unsigned*) alignedUSMMalloc(width*height*sizeof(unsigned),64,EmbreeUSMMode::DEVICE_READ_WRITE);
   }
 
   void TutorialApplication::set_scene (TutorialScene* in)
