@@ -12,22 +12,16 @@
 
 RTC_NAMESPACE_USE
 
-#include <xmmintrin.h>
-//#include <pmmintrin.h> // use this to get _MM_SET_DENORMALS_ZERO_MODE when compiling for SSE3 or higher
-
-#if !defined(_MM_SET_DENORMALS_ZERO_MODE)
-#define _MM_DENORMALS_ZERO_ON   (0x0040)
-#define _MM_DENORMALS_ZERO_OFF  (0x0000)
-#define _MM_DENORMALS_ZERO_MASK (0x0040)
-#define _MM_SET_DENORMALS_ZERO_MODE(x) (_mm_setcsr((_mm_getcsr() & ~_MM_DENORMALS_ZERO_MASK) | (x)))
-#endif
-
 int main() {
-  /* for best performance set FTZ and DAZ flags in MXCSR control and status register */
-  _MM_SET_FLUSH_ZERO_MODE(_MM_FLUSH_ZERO_ON);
-  _MM_SET_DENORMALS_ZERO_MODE(_MM_DENORMALS_ZERO_ON);
 
 #if defined(EMBREE_SYCL_SUPPORT)
+  try {
+    embree::check_raytracing_support();
+  } catch (std::exception& e) {
+    std::cerr << e.what() << std::endl;
+    return 1;
+  }
+
   try {
     sycl::device sycl_device(rtcSYCLDeviceSelector);
     sycl::context sycl_context(sycl_device);
