@@ -4,12 +4,20 @@
 #include "../common/tutorial/tutorial.h"
 #include "../common/tutorial/benchmark_render.h"
 
+#if defined(EMBREE_SYCL_TUTORIAL)
+#  define NAME "grid_geometry_sycl"
+#  define FEATURES FEATURE_RTCORE | FEATURE_SYCL
+#else
+#  define NAME "grid_geometry"
+#  define FEATURES FEATURE_RTCORE
+#endif
+
 namespace embree
 {
-  struct Tutorial : public TutorialApplication 
+  struct Tutorial : public TutorialApplication
   {
     Tutorial()
-      : TutorialApplication("grid_geometry",FEATURE_RTCORE) 
+        : TutorialApplication(NAME, FEATURES)
     {
       /* set default camera */
       camera.from = Vec3fa(1.5f,1.5f,-1.5f);
