@@ -5,10 +5,20 @@
 
 namespace embree {
 
+/* all features required by this tutorial */
+#define FEATURE_MASK \
+  RTC_FEATURE_FLAG_TRIANGLE | \
+  RTC_FEATURE_FLAG_INSTANCE | \
+  RTC_FEATURE_FLAG_32_BIT_RAY_MASK
+
 const int numPhi = 5;
 const int numTheta = 2*numPhi;
 
 RTCScene g_scene  = nullptr;
+RTCGeometry g_instance0 = nullptr;
+RTCGeometry g_instance1 = nullptr;
+RTCGeometry g_instance2 = nullptr;
+RTCGeometry g_instance3 = nullptr;
 TutorialData data;
 
 unsigned int createTriangulatedSphere (RTCScene scene, const Vec3fa& p, float r)
@@ -110,61 +120,63 @@ extern "C" void device_init (char* cfg)
   rtcCommitScene (data.g_scene1);
 
   /* instantiate geometry */
-  data.g_instance0 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
-  rtcSetGeometryInstancedScene(data.g_instance0,data.g_scene1);
-  rtcSetGeometryTimeStepCount(data.g_instance0,1);
-  data.g_instance1 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
-  rtcSetGeometryInstancedScene(data.g_instance1,data.g_scene1);
-  rtcSetGeometryTimeStepCount(data.g_instance1,1);
-  data.g_instance2 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
-  rtcSetGeometryInstancedScene(data.g_instance2,data.g_scene1);
-  rtcSetGeometryTimeStepCount(data.g_instance2,1);
-  data.g_instance3 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
-  rtcSetGeometryInstancedScene(data.g_instance3,data.g_scene1);
-  rtcSetGeometryTimeStepCount(data.g_instance3,1);
-  rtcAttachGeometry(data.g_scene,data.g_instance0);
-  rtcAttachGeometry(data.g_scene,data.g_instance1);
-  rtcAttachGeometry(data.g_scene,data.g_instance2);
-  rtcAttachGeometry(data.g_scene,data.g_instance3);
-  rtcReleaseGeometry(data.g_instance0);
-  rtcReleaseGeometry(data.g_instance1);
-  rtcReleaseGeometry(data.g_instance2);
-  rtcReleaseGeometry(data.g_instance3);
+  g_instance0 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
+  rtcSetGeometryMask(g_instance0, 0x80); // test high instance mask bits
+  rtcSetGeometryInstancedScene(g_instance0,data.g_scene1);
+  rtcSetGeometryTimeStepCount(g_instance0,1);
+  g_instance1 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
+  rtcSetGeometryInstancedScene(g_instance1,data.g_scene1);
+  rtcSetGeometryTimeStepCount(g_instance1,1);
+  g_instance2 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
+  rtcSetGeometryInstancedScene(g_instance2,data.g_scene1);
+  rtcSetGeometryTimeStepCount(g_instance2,1);
+  g_instance3 = rtcNewGeometry (g_device, RTC_GEOMETRY_TYPE_INSTANCE);
+  rtcSetGeometryInstancedScene(g_instance3,data.g_scene1);
+  rtcSetGeometryTimeStepCount(g_instance3,1);
+  rtcAttachGeometry(data.g_scene,g_instance0);
+  rtcAttachGeometry(data.g_scene,g_instance1);
+  rtcAttachGeometry(data.g_scene,g_instance2);
+  rtcAttachGeometry(data.g_scene,g_instance3);
+  rtcReleaseGeometry(g_instance0);
+  rtcReleaseGeometry(g_instance1);
+  rtcReleaseGeometry(g_instance2);
+  rtcReleaseGeometry(g_instance3);
   createGroundPlane(data.g_scene);
 
   /* set all colors */
-  data.colors[0][0] = Vec3fa(0.25f, 0.f, 0.f);
-  data.colors[0][1] = Vec3fa(0.50f, 0.f, 0.f);
-  data.colors[0][2] = Vec3fa(0.75f, 0.f, 0.f);
-  data.colors[0][3] = Vec3fa(1.00f, 0.f, 0.f);
+  data.colors[4*0+0] = Vec3fa(0.25f, 0.f, 0.f);
+  data.colors[4*0+1] = Vec3fa(0.50f, 0.f, 0.f);
+  data.colors[4*0+2] = Vec3fa(0.75f, 0.f, 0.f);
+  data.colors[4*0+3] = Vec3fa(1.00f, 0.f, 0.f);
 
-  data.colors[1][0] = Vec3fa(0.f, 0.25f, 0.f);
-  data.colors[1][1] = Vec3fa(0.f, 0.50f, 0.f);
-  data.colors[1][2] = Vec3fa(0.f, 0.75f, 0.f);
-  data.colors[1][3] = Vec3fa(0.f, 1.00f, 0.f);
+  data.colors[4*1+0] = Vec3fa(0.f, 0.25f, 0.f);
+  data.colors[4*1+1] = Vec3fa(0.f, 0.50f, 0.f);
+  data.colors[4*1+2] = Vec3fa(0.f, 0.75f, 0.f);
+  data.colors[4*1+3] = Vec3fa(0.f, 1.00f, 0.f);
 
-  data.colors[2][0] = Vec3fa(0.f, 0.f, 0.25f);
-  data.colors[2][1] = Vec3fa(0.f, 0.f, 0.50f);
-  data.colors[2][2] = Vec3fa(0.f, 0.f, 0.75f);
-  data.colors[2][3] = Vec3fa(0.f, 0.f, 1.00f);
+  data.colors[4*2+0] = Vec3fa(0.f, 0.f, 0.25f);
+  data.colors[4*2+1] = Vec3fa(0.f, 0.f, 0.50f);
+  data.colors[4*2+2] = Vec3fa(0.f, 0.f, 0.75f);
+  data.colors[4*2+3] = Vec3fa(0.f, 0.f, 1.00f);
 
-  data.colors[3][0] = Vec3fa(0.25f, 0.25f, 0.f);
-  data.colors[3][1] = Vec3fa(0.50f, 0.50f, 0.f);
-  data.colors[3][2] = Vec3fa(0.75f, 0.75f, 0.f);
-  data.colors[3][3] = Vec3fa(1.00f, 1.00f, 0.f);
+  data.colors[4*3+0] = Vec3fa(0.25f, 0.25f, 0.f);
+  data.colors[4*3+1] = Vec3fa(0.50f, 0.50f, 0.f);
+  data.colors[4*3+2] = Vec3fa(0.75f, 0.75f, 0.f);
+  data.colors[4*3+3] = Vec3fa(1.00f, 1.00f, 0.f);
 }
 
 /* task that renders a single screen tile */
-Vec3fa renderPixelStandard(const TutorialData& data, float x, float y, const ISPCCamera& camera, RayStats& stats)
+Vec3fa renderPixel(const TutorialData& data, float x, float y, const ISPCCamera& camera, RayStats& stats)
 {
-  RTCIntersectContext context;
-  rtcInitIntersectContext(&context);
-  
   /* initialize ray */
   Ray ray(Vec3fa(camera.xfm.p), Vec3fa(normalize(x*camera.xfm.l.vx + y*camera.xfm.l.vy + camera.xfm.l.vz)), 0.0f, inf);
 
   /* intersect ray with scene */
-  rtcIntersect1(data.g_scene,&context,RTCRayHit_(ray));
+  RTCIntersectArguments iargs;
+  rtcInitIntersectArguments(&iargs);
+  iargs.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
+  
+  rtcTraversableIntersect1(data.g_traversable,RTCRayHit_(ray),&iargs);
   RayStats_addRay(stats);
 
   /* shade pixels */
@@ -174,13 +186,18 @@ Vec3fa renderPixelStandard(const TutorialData& data, float x, float y, const ISP
     /* calculate shading normal in world space */
     Vec3fa Ns = ray.Ng;
     if (ray.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      Ns = xfmVector(data.normal_xfm[ray.instID[0]],Ns);
+    {
+      AffineSpace3fa xfm;
+      rtcGetGeometryTransformFromTraversable(data.g_traversable,ray.instID[0],0.0f,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,&xfm);
+      Ns = xfmNormal(xfm,Ns);
+      //Ns = xfmVector(data.normal_xfm[ray.instID[0]],Ns);
+    }
     Ns = normalize(Ns);
 
     /* calculate diffuse color of geometries */
     Vec3fa diffuse = Vec3fa(1,1,1);
     if (ray.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      diffuse = data.colors[ray.instID[0]][ray.geomID];
+      diffuse = data.colors[4*ray.instID[0]+ray.geomID];
     color = color + diffuse*0.5;
 
     /* initialize shadow ray */
@@ -188,7 +205,11 @@ Vec3fa renderPixelStandard(const TutorialData& data, float x, float y, const ISP
     Ray shadow(ray.org + ray.tfar*ray.dir, neg(lightDir), 0.001f, inf);
 
     /* trace shadow ray */
-    rtcOccluded1(data.g_scene,&context,RTCRay_(shadow));
+    RTCOccludedArguments sargs;
+    rtcInitOccludedArguments(&sargs);
+    sargs.feature_mask = (RTCFeatureFlags) (FEATURE_MASK);
+    
+    rtcTraversableOccluded1(data.g_traversable,RTCRay_(shadow),&sargs);
     RayStats_addShadowRay(stats);
 
     /* add light contribution */
@@ -198,6 +219,24 @@ Vec3fa renderPixelStandard(const TutorialData& data, float x, float y, const ISP
   return color;
 }
 
+void renderPixelStandard(const TutorialData& data,
+                         int x, int y, 
+                         int* pixels,
+                         const unsigned int width,
+                         const unsigned int height,
+                         const float time,
+                         const ISPCCamera& camera, RayStats& stats)
+{
+  /* calculate pixel color */
+  Vec3fa color = renderPixel(data, (float)x,(float)y,camera, stats);
+  
+  /* write color to framebuffer */
+  unsigned int r = (unsigned int) (255.0f * clamp(color.x,0.0f,1.0f));
+  unsigned int g = (unsigned int) (255.0f * clamp(color.y,0.0f,1.0f));
+  unsigned int b = (unsigned int) (255.0f * clamp(color.z,0.0f,1.0f));
+  pixels[y*width+x] = (b << 16) + (g << 8) + r;
+}
+
 /* renders a single screen tile */
 void renderTileStandard(int taskIndex,
                         int threadIndex,
@@ -218,171 +257,7 @@ void renderTileStandard(int taskIndex,
 
   for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
   {
-    /* calculate pixel color */
-    Vec3fa color = renderPixelStandard(data, (float)x,(float)y,camera,g_stats[threadIndex]);
-
-    /* write color to framebuffer */
-    unsigned int r = (unsigned int) (255.0f * clamp(color.x,0.0f,1.0f));
-    unsigned int g = (unsigned int) (255.0f * clamp(color.y,0.0f,1.0f));
-    unsigned int b = (unsigned int) (255.0f * clamp(color.z,0.0f,1.0f));
-    pixels[y*width+x] = (b << 16) + (g << 8) + r;
-  }
-}
-
-/* renders a single screen tile */
-void renderTileStandardStream(int taskIndex,
-                              int threadIndex,
-                              int* pixels,
-                              const unsigned int width,
-                              const unsigned int height,
-                              const float time,
-                              const ISPCCamera& camera,
-                              const int numTilesX,
-                              const int numTilesY)
-{
-  const unsigned int tileY = taskIndex / numTilesX;
-  const unsigned int tileX = taskIndex - tileY * numTilesX;
-  const unsigned int x0 = tileX * TILE_SIZE_X;
-  const unsigned int x1 = min(x0+TILE_SIZE_X,width);
-  const unsigned int y0 = tileY * TILE_SIZE_Y;
-  const unsigned int y1 = min(y0+TILE_SIZE_Y,height);
-
-  RayStats& stats = g_stats[threadIndex];
-
-  Ray primary_stream[TILE_SIZE_X*TILE_SIZE_Y];
-  Ray shadow_stream[TILE_SIZE_X*TILE_SIZE_Y];
-  Vec3fa color_stream[TILE_SIZE_X*TILE_SIZE_Y];
-  bool valid_stream[TILE_SIZE_X*TILE_SIZE_Y];
-
-  /* generate stream of primary rays */
-  int N = 0;
-  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
-  {
-    /* ISPC workaround for mask == 0 */
-    
-
-    /* initialize variables */
-    color_stream[N] = Vec3fa(0.0f);
-    bool mask = 1; { valid_stream[N] = mask; }
-
-    /* initialize ray */
-    Ray& primary = primary_stream[N];
-    mask = 1; { // invalidates inactive rays
-      primary.tnear() = mask ? 0.0f         : (float)(pos_inf);
-      primary.tfar  = mask ? (float)(inf) : (float)(neg_inf);
-    }
-    init_Ray(primary, Vec3fa(camera.xfm.p), Vec3fa(normalize((float)x*camera.xfm.l.vx + (float)y*camera.xfm.l.vy + camera.xfm.l.vz)), primary.tnear(), primary.tfar);
-
-    N++;
-    RayStats_addRay(stats);
-  }
-
-  Vec3fa lightDir = normalize(Vec3fa(-1,-1,-1));
-
-  /* trace rays */
-  RTCIntersectContext primary_context;
-  rtcInitIntersectContext(&primary_context);
-  primary_context.flags = g_iflags_coherent;
-  rtcIntersect1M(data.g_scene,&primary_context,(RTCRayHit*)&primary_stream,N,sizeof(Ray));
-
-  /* terminate rays and update color */
-  N = -1;
-  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
-  {
-    N++;
-    /* ISPC workaround for mask == 0 */
-    
-
-    /* invalidate shadow rays by default */
-    Ray& shadow = shadow_stream[N];
-    {
-      shadow.tnear() = (float)(pos_inf);
-      shadow.tfar  = (float)(neg_inf);
-    }
-
-    /* ignore invalid rays */
-    if (valid_stream[N] == false) continue;
-
-    /* terminate rays that hit nothing */
-    if (primary_stream[N].geomID == RTC_INVALID_GEOMETRY_ID) {
-      valid_stream[N] = false;
-      continue;
-    }
-
-    /* calculate shading normal in world space */
-    Ray& primary = primary_stream[N];
-    Vec3fa Ns = primary.Ng;
-    if (primary.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      Ns = xfmVector(data.normal_xfm[primary.instID[0]],Ns);
-    Ns = normalize(Ns);
-
-    /* calculate diffuse color of geometries */
-    Vec3fa diffuse = Vec3fa(1,1,1);
-    if (primary.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      diffuse = data.colors[primary.instID[0]][primary.geomID];
-    color_stream[N] = color_stream[N] + diffuse*0.5;
-
-    /* initialize shadow ray tnear/tfar */
-    bool mask = 1; {
-      shadow.tnear() = mask ? 0.001f       : (float)(pos_inf);
-      shadow.tfar  = mask ? (float)(inf) : (float)(neg_inf);
-    }
-    init_Ray(shadow, primary.org + primary.tfar*primary.dir, neg(lightDir), shadow.tnear(), shadow.tfar);
-
-    RayStats_addShadowRay(stats);
-  }
-  N++;
-
-  /* trace shadow rays */
-  RTCIntersectContext shadow_context;
-  rtcInitIntersectContext(&shadow_context);
-  shadow_context.flags = g_iflags_coherent;
-  rtcOccluded1M(data.g_scene,&shadow_context,(RTCRay*)&shadow_stream,N,sizeof(Ray));
-
-  /* add light contribution */
-  N = -1;
-  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
-  {
-    N++;
-    /* ISPC workaround for mask == 0 */
-    
-
-    /* ignore invalid rays */
-    if (valid_stream[N] == false) continue;
-
-    /* calculate shading normal in world space */
-    Ray& primary = primary_stream[N];
-    Vec3fa Ns = primary.Ng;
-    if (primary.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      Ns = xfmVector(data.normal_xfm[primary.instID[0]],Ns);
-    Ns = normalize(Ns);
-
-    /* calculate diffuse color of geometries */
-    Vec3fa diffuse = Vec3fa(1,1,1);
-    if (primary.instID[0] != RTC_INVALID_GEOMETRY_ID)
-      diffuse = data.colors[primary.instID[0]][primary.geomID];
-
-    /* add light contribution */
-    Ray& shadow = shadow_stream[N];
-    if (shadow.tfar >= 0.0f) {
-      color_stream[N] = color_stream[N] + diffuse*clamp(-dot(lightDir,Ns),0.0f,1.0f);
-    }
-  }
-  N++;
-
-  /* framebuffer writeback */
-  N = 0;
-  for (unsigned int y=y0; y<y1; y++) for (unsigned int x=x0; x<x1; x++)
-  {
-    /* ISPC workaround for mask == 0 */
-    
-
-    /* write color to framebuffer */
-    unsigned int r = (unsigned int) (255.0f * clamp(color_stream[N].x,0.0f,1.0f));
-    unsigned int g = (unsigned int) (255.0f * clamp(color_stream[N].y,0.0f,1.0f));
-    unsigned int b = (unsigned int) (255.0f * clamp(color_stream[N].z,0.0f,1.0f));
-    pixels[y*width+x] = (b << 16) + (g << 8) + r;
-    N++;
+    renderPixelStandard(data,x,y,pixels,width,height,time,camera,g_stats[threadIndex]);
   }
 }
 
@@ -395,10 +270,7 @@ void renderTileTask (int taskIndex, int threadIndex, int* pixels,
                                  const int numTilesX,
                                  const int numTilesY)
 {
-  if (g_mode == MODE_NORMAL)
-    renderTileStandard(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
-  else
-    renderTileStandardStream(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
+  renderTileStandard(taskIndex,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
 }
 
 extern "C" void renderFrameStandard (int* pixels,
@@ -408,6 +280,25 @@ extern "C" void renderFrameStandard (int* pixels,
                           const ISPCCamera& camera)
 {
   /* render all pixels */
+#if defined(EMBREE_SYCL_TUTORIAL) && !defined(EMBREE_SYCL_RT_SIMULATION)
+  TutorialData ldata = data;
+  sycl::event event = global_gpu_queue->submit([=](sycl::handler& cgh){
+    const sycl::nd_range<2> nd_range = make_nd_range(height,width);
+    cgh.parallel_for(nd_range,[=](sycl::nd_item<2> item) {
+      const unsigned int x = item.get_global_id(1); if (x >= width ) return;
+      const unsigned int y = item.get_global_id(0); if (y >= height) return;
+      RayStats stats;
+      renderPixelStandard(ldata,x,y,pixels,width,height,time,camera,stats);
+    });
+  });
+  global_gpu_queue->wait_and_throw();
+
+  const auto t0 = event.template get_profiling_info<sycl::info::event_profiling::command_start>();
+  const auto t1 = event.template get_profiling_info<sycl::info::event_profiling::command_end>();
+  const double dt = (t1-t0)*1E-9;
+  ((ISPCCamera*)&camera)->render_time = dt;
+  
+#else
   const int numTilesX = (width +TILE_SIZE_X-1)/TILE_SIZE_X;
   const int numTilesY = (height+TILE_SIZE_Y-1)/TILE_SIZE_Y;
   parallel_for(size_t(0),size_t(numTilesX*numTilesY),[&](const range<size_t>& range) {
@@ -415,6 +306,7 @@ extern "C" void renderFrameStandard (int* pixels,
     for (size_t i=range.begin(); i<range.end(); i++)
       renderTileTask((int)i,threadIndex,pixels,width,height,time,camera,numTilesX,numTilesY);
   }); 
+#endif
 }
 
 /* called by the C++ code to render */
@@ -444,17 +336,18 @@ extern "C" void device_render (int* pixels,
     data.normal_xfm[i] = transposed(rcp(data.instance_xfm[i].l));
 
   /* set instance transformations */
-  rtcSetGeometryTransform(data.g_instance0,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[0]);
-  rtcSetGeometryTransform(data.g_instance1,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[1]);
-  rtcSetGeometryTransform(data.g_instance2,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[2]);
-  rtcSetGeometryTransform(data.g_instance3,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[3]);
+  rtcSetGeometryTransform(g_instance0,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[0]);
+  rtcSetGeometryTransform(g_instance1,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[1]);
+  rtcSetGeometryTransform(g_instance2,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[2]);
+  rtcSetGeometryTransform(g_instance3,0,RTC_FORMAT_FLOAT4X4_COLUMN_MAJOR,(float*)&data.instance_xfm[3]);
 
   /* update scene */
-  rtcCommitGeometry(data.g_instance0);
-  rtcCommitGeometry(data.g_instance1);
-  rtcCommitGeometry(data.g_instance2);
-  rtcCommitGeometry(data.g_instance3);
+  rtcCommitGeometry(g_instance0);
+  rtcCommitGeometry(g_instance1);
+  rtcCommitGeometry(g_instance2);
+  rtcCommitGeometry(g_instance3);
   rtcCommitScene (data.g_scene);
+  data.g_traversable = rtcGetSceneTraversable(data.g_scene);
 }
 
 /* called by the C++ code for cleanup */
