@@ -8,19 +8,26 @@ namespace embree {
 struct TutorialData
 {
   RTCScene g_scene;
+  RTCTraversable g_traversable;
   Vec3fa* colors;
+
+  RTCFilterFunctionN intersectionFilter;
+  RTCFilterFunctionN occlusionFilter;
 };
 
 inline void TutorialData_Constructor(TutorialData* This)
 {
   This->g_scene = nullptr;
+  This->g_traversable = nullptr;
   This->colors = nullptr;
+  This->intersectionFilter = nullptr;
+  This->occlusionFilter = nullptr;
 }
 
 inline void TutorialData_Destructor(TutorialData* This)
 {
   rtcReleaseScene (This->g_scene); This->g_scene = nullptr;
-  alignedFree(This->colors); This->colors = nullptr;
+  alignedUSMFree(This->colors); This->colors = nullptr;
 }
 
 } // namespace embree
