@@ -4,12 +4,20 @@
 #include "../common/tutorial/tutorial.h"
 #include "../common/tutorial/benchmark_render.h"
 
+#if defined(EMBREE_SYCL_TUTORIAL)
+#  define NAME "point_geometry_sycl"
+#  define FEATURES FEATURE_RTCORE | FEATURE_SYCL
+#else
+#  define NAME "point_geometry"
+#  define FEATURES FEATURE_RTCORE
+#endif
+
 namespace embree
 {
   struct Tutorial : public TutorialApplication
   {
     Tutorial()
-      : TutorialApplication("point_geometry",FEATURE_RTCORE)
+      : TutorialApplication(NAME,FEATURES)
     {
       /* set default camera */
       camera.from = Vec3fa(0.0f, 2.0f, 7.0f);
