@@ -8,6 +8,7 @@ namespace embree {
 struct TutorialData
 {
   RTCScene g_scene;
+  RTCTraversable g_traversable;
   Vec3fa* face_colors;
   Vec3fa* vertex_colors;
 };
@@ -15,6 +16,7 @@ struct TutorialData
 inline void TutorialData_Constructor(TutorialData* This)
 {
   This->g_scene = nullptr;
+  This->g_traversable = nullptr;
   This->face_colors = nullptr;
   This->vertex_colors = nullptr;
 }
