@@ -5,9 +5,16 @@
 
 namespace embree {
 
+enum UserGeometryType
+{
+  USER_GEOMETRY_INSTANCE = 0,
+  USER_GEOMETRY_SPHERE = 1
+};
+
 struct Sphere
 {
   ALIGNED_STRUCT_(16)
+  UserGeometryType type;
   Vec3fa p;                      //!< position of the sphere
   float r;                      //!< radius of the sphere
   RTCGeometry geometry;
@@ -17,9 +24,9 @@ struct Sphere
 struct Instance
 {
   ALIGNED_STRUCT_(16)
+  UserGeometryType type;
   RTCGeometry geometry;
-  RTCScene object;
-  int userID;
+  RTCTraversable object;
   AffineSpace3fa local2world;
   AffineSpace3fa world2local;
   LinearSpace3fa normal2world;
@@ -34,21 +41,34 @@ struct TutorialData
   RTCScene g_scene0;
   RTCScene g_scene1;
   RTCScene g_scene2;
+
+  RTCTraversable g_traversable;
+
   Sphere* g_spheres;
   Sphere* g_sphere0;
   Sphere* g_sphere1;
-  
+
   Instance* g_instance[4];
-  
-  Vec3fa colors[5][4];
+
+  Vec3fa* colors;
 };
 
+#if __SYCL_COMPILER_VERSION >= 20210801
+}
+namespace sycl {
+  template<> struct is_device_copyable<embree::TutorialData> : std::true_type {};
+  template<> struct is_device_copyable<const embree::TutorialData> : std::true_type {};
+}
+namespace embree {
+#endif
+
 inline void TutorialData_Constructor(TutorialData* This)
 {
   This->g_scene  = nullptr;
   This->g_scene0 = nullptr;
   This->g_scene1 = nullptr;
   This->g_scene2 = nullptr;
+  This->g_traversable = nullptr;
   This->g_spheres = nullptr;
   This->g_sphere0 = nullptr;
   This->g_sphere1 = nullptr;
@@ -56,6 +76,7 @@ inline void TutorialData_Constructor(TutorialData* This)
   This->g_instance[1] = nullptr;
   This->g_instance[2] = nullptr;
   This->g_instance[3] = nullptr;
+  This->colors = (Vec3fa*) alignedUSMMalloc((5*4)*sizeof(Vec3fa),16);
 }
 
 inline void TutorialData_Destructor(TutorialData* This)
@@ -65,9 +86,10 @@ inline void TutorialData_Destructor(TutorialData* This)
   rtcReleaseScene (This->g_scene1); This->g_scene1 = nullptr;
   rtcReleaseScene (This->g_scene2); This->g_scene2 = nullptr;
   rtcReleaseDevice(g_device); g_device = nullptr;
-  alignedFree(This->g_spheres); This->g_spheres = nullptr;
-  alignedFree(This->g_sphere0); This->g_sphere0 = nullptr;
-  alignedFree(This->g_sphere1); This->g_sphere1 = nullptr;
+  alignedUSMFree(This->g_spheres); This->g_spheres = nullptr;
+  alignedUSMFree(This->g_sphere0); This->g_sphere0 = nullptr;
+  alignedUSMFree(This->g_sphere1); This->g_sphere1 = nullptr;
+  alignedUSMFree(This->colors); This->colors = nullptr;
 }
 
 } // namespace embree
